require 'thor'
require 'stomp_job'

module StompJob
  class Cli < Thor
    BANNER = <<-EOF
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMM   STOMPJOB   MMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMNmddmNMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMmy+:.       `/NMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMmo.      `+.   `+mMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMd         -smmydNhMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMM.        .odNydNhMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMo        `o:   `sMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMm        ::`  `+mMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMM-       ohNdhMhmMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMo       :yNdsdNMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMm       +o.  -hmhso++oymMMMMMMMMMMMMMMMM
MMMMMMMMMMM-         `//`         .oNMMMMMMMMMMMMM
MMMMMMMMMMMs                        .mMMMMMMMMMMMM
MMMMMMMMMMMN                         .MMMMMMMMMMMM
MMMMMMMMMMMMNdyo+:-.`           `-:ohNNMMMMMMMMMMM
MMMMMMMMMMMM`.:+oydMNNNNmmmmmNNNmhyo:`+MMMMMMMMMMM
MMMMh:``dMMMso+/:-mMN:    ``    `.:+smMMMy``/dMMMM
MMds//-  ./oshdmNNMMMMMMNNNmNNNMNNmdhso/.  -//sdMM
MMMMMm- `/o-       ```......``        :o:` -NMMMMM
MMMMd/sdMMM/  `oNM/    :md:    :MNo`  +MMMds/dMMMM
MMMMMMMMMMM``sNMMMy   oMMMMs`  oMMMNs``MMMMMMMMMMM
MMMMMMMMMMdsMMMMMMN .dMMMMMMm- yMMMMMNsmMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMsNMMMMMMMMMomMMMMMMMMMMMMMMMMMM
    EOF

    default_task :work


    desc "work", "Runs all registered workers"
    method_option :require,
      aliases: ['-r'],
      banner: 'REQUIRED_WORKERS',
      desc: 'REQUIRED_WORKERS is a comma separated list of files to require'
    method_option :rails,
      desc: 'Make StompJob try to load the rails environment.'

    def work
      say BANNER
      say
      require_files if options['require']
      load_rails if options['rails']

      say "Starting Stomper with workers:"
      say pretty_print_workers, :yellow
      say

      Workers.listen!
    end


    private

    def require_files
      options['require'].split(',').each do |require_file|
        require require_file
      end
    rescue Exception => e
      die(e)
    end

    def load_rails
      require './config/application'
      ::Rails.application.eager_load!
    rescue Exception => e
      die(e, "Are you running this from a rails app?\nStompJob expects to be able to `require ./config/application`")
    end


    def die(exception, debug_help=nil)
      say "StompJob encountered the following error and is quitting:\n\t#{exception.message}", :red
      say
      say debug_help, :red if debug_help

      exit 1
    end

    def pretty_print_workers
      Workers.registered_workers.map{|worker| "\t#{worker.name}\n"}.join('')
    end
  end
end
