module StompJob
  class Configuration
    DEFAULT_CLIENT_CONFIG = {
      hosts: [
        {login: "", passcode: "", host: "localhost", port: 61613}
      ],
      reliable: true,
      initial_reconnect_delay: 0.01,
      max_reconnect_delay: 30.0,
      use_exponential_back_off: true,
      back_off_multiplier: 2,
      max_reconnect_attempts: 0,
      randomize: false,
      connect_timeout: 0,
      connect_headers: {
        "accept-version" => "1.1",
        "host" => "localhost",
        "heart-beat" => "60000,60000"
      },
      parse_timeout: 5,
      logger: nil,
      dmh: false,
      closed_check: true,
      hbser: true,
      stompconn: false,
      usecrlf: false,
      max_hbread_fails: 0,
      max_hbrlck_fails: 0,
      fast_hbs_adjust: 20.0,
      tcp_nodelay: true,
      start_timeout: 0,
      sslctx_newparm: nil,
      ssl_post_conn_check: true
    }


    def self.configuration
      @configuration ||= DEFAULT_CLIENT_CONFIG
    end

    def self.reset_configuration!
      @configuration = DEFAULT_CLIENT_CONFIG
    end

    def self.setup
      yield configuration
    end

    def self.load_config!(file="config/stomp_job.yml")
      raise 'YAML undefined! You must require YAML to use the StompJob::Configuration.load_config! method!' unless defined?(YAML)
      reset_configuration!
      configuration.merge! deep_symbolize_keys(YAML.load_file(file))
    end

    private

    def self.deep_symbolize_keys(object)
      case
      when object.is_a?(Array)
        object.map { |element| deep_symbolize_keys(element) }
      when object.is_a?(Hash)
        object.inject({}) do |memo, (key,value)|
          memo[key.to_sym] = deep_symbolize_keys(value)
          memo
        end
      else
        object
      end
    end
  end
end
