require "stomp"

module StompJob
  class Connection

    def self.from(stomp_configuration={})
      new(new_client( Configuration.configuration.merge(stomp_configuration) ))
    end

    def self.default_connection
      @default_connection ||= refresh_default_connection
    end

    def self.refresh_default_connection
      @default_connection = new
    end



    # Using a marshaller allows us to send complex messages over stomp.
    # Using Ruby's built-in `Marshal` by default.
    def publish(queue_name, *args)
      @connection.publish(safe_queue_name(queue_name), marshaller.dump(args))
    end

    def subscribe(worker_klass)
      @connection.subscribe( safe_queue_name(worker_klass.queue_name) ) do |msg|
        worker_klass.new.perform( *marshaller.load(msg.body) )
      end
    end


    private

    def self.new_client(config)
      Stomp::Client.new(config)
    rescue RuntimeError => exception # All STOMP gem errors subclass RuntimeError
      # We wrap the STOMP exception with our own to provide an extra layer of
      # indirection.
      raise ConnectionException.new(exception)
    end

    def initialize(connection=nil)
      @connection = connection || self.class.new_client( Configuration.configuration )
    end

    # The STOMP gem can break the interpreter if the queue name is invalid.
    # This method ensures that queue_names are properly formatted.
    def safe_queue_name(queue_name)
      queue_name.match(/\Ajms\.queue/) ? queue_name : "jms.queue.#{queue_name}"
    end

    def marshaller
      Marshal
    end

  end
end
