require 'stomp_job'

module ActiveJob
  module QueueAdapters
    # == StompJob adapter for Active Job
    #
    # StompJob is an ActiveJob adapter for working with any message broker that
    # accepts the STOMP protocol.
    #
    # To use StompJob set the queue_adapter config to +:stomp_job+.
    #
    #   Rails.application.config.active_job.queue_adapter = :stomp_job
    class StompJobAdapter
      def enqueue(job)
        StompJobAdapter.stomp_worker_from_aj(job.class).enqueue(*job.arguments)
      end

      def enqueue_at(job, timestamp)
        raise NotImplementedError, "Scheduling jobs is not currently supported by the StopmJob queueing backend."
      end


      def self.stomp_worker_from_aj(aj_klass)
        stomp_job_klasses[aj_klass] ||= Class.new do
          include StompJob::Worker
          stomp_options queue_name: aj_klass.queue_name
        end
      end

      def self.stomp_job_klasses
        @stomp_job_klasses ||= {}
      end
    end
  end

  class Base
    # It's necessary to populate StompJobAdapter.stomp_job_klasses when the jobs
    # are created, not when they're enqueued, or else listeners won't know that
    # the anonymous StompJob classes exist.
    #
    # Overriding queue_as allows us to do this.
    def self.queue_as(*args)
      super
      QueueAdapters::StompJobAdapter.stomp_worker_from_aj(self)
    end
  end
end
